#include "c_interface.hpp"

/* ********************************************************** */
/*                      CONVERSION FUNCTION                   */
/* ********************************************************** */

static inline std::string stringXtoStd(XString _str)
{
   char * const temp = new char[_str.len+1]();
   memset(temp, '\0', _str.len+1);
   memcpy(temp, _str.str, _str.len);
   std::string _retstr(temp);
   size_t d = _retstr.find_first_not_of(' ');
   size_t f = _retstr.find_last_not_of (' ');

   delete[] temp;
   return (_retstr.substr(d, f-d+1));
}

static inline XString stringXtoStd(const std::string& _str)
{
   XString _retstr = {new char[_str.size()](), _str.size()};
   memcpy(_retstr.str, _str.c_str(), _str.size());
   return (_retstr);
}

/* ********************************************************** */
/*                      HANDLE INTERFACE                      */
/* ********************************************************** */

void xios_handle_create_(XHandle * _ret, const XDType * const _dtype, XString _id)
{
   // Si le handle n'est pas initialisé à 0, on ne fait aucun traitement.
   if (!isNullHandle(*_ret)) return;
   std::string __id = stringXtoStd(_id);
   switch(*_dtype)
   {
      /* EAXIS, EDOMAIN, EFIELD, EFILE, EGRID, GAXIS, GDOMAIN, GFIELD, GFILE, GGRID*/
      case (ECONTEXT):
         // Si le context n'existe pas, on retourne un handle vide/nul.
         if (!Context::HasContext(__id)) { *_ret = NULLHANDLE; return ; }
         _ret->data_type = *_dtype;
         _ret->data_ptr  = Context::GetContext(__id);
         return ;

      default :
         *_ret = NULLHANDLE;
         return ;
   };
}

/* ********************************************************** */
/*                      XML INTERFACE                         */
/* ********************************************************** */

void xios_xml_parse_file_(XString _filename)
{
   std::string   __filename = stringXtoStd(_filename);
   std::ifstream __istr( __filename.c_str() , std::ifstream::in );

   // On commence la lecture du flux de donnée xml.
   XMLNode node = XMLNode::CreateNode(__istr, Context::GetRootName());

   // On parse le fichier xml noeud par noeud
   // (ie on construit dynamiquement notre arbre d'objets).
   XMLParser::Parse(node);
}

void xios_xml_parse_string_(XString _xmlcontent)
{
   std::string        __xmlcontent = stringXtoStd(_xmlcontent);
   std::istringstream __istr(__xmlcontent);

   // On commence la lecture du flux de donnée xml.
   XMLNode node = XMLNode::CreateNode(__istr, Context::GetRootName());

   // On parse le fichier xml noeud par noeud
   // (ie on construit dynamiquement notre arbre d'objets).
   XMLParser::Parse(node);
}
/* ********************************************************** */
/*                      DATA TREATMENT INTERFACE              */
/* ********************************************************** */

static std::vector<DataTreatment *> AllDataTreatment;

static void deleteAllDataTreatment(void)
{
   std::vector<DataTreatment *>::iterator it;
   for (it  = AllDataTreatment.begin();
        it != AllDataTreatment.end(); it++)
      delete (*it);
}

void xios_dtreatment_new_(XHandle * _dt, const XHandle * const _hd)
{
   static bool called = false;

   // Si le handle de traitement n'est pas initialisé à 0, on ne fait aucun traitement.
   if (!isNullHandle(*_dt) || (_hd->data_type != ECONTEXT)) return;
   Context* const __ctxt = (Context*) _hd->data_ptr;

   _dt->data_type = DTREATMENT;
   _dt->data_ptr  = new DataTreatment(__ctxt);

   if (!called)
   {
      atexit (&deleteAllDataTreatment);
      called = true;
   }
}

void xios_dtreatment_create_files_and_headers_(const XHandle * const _hd, const XFileType * const _ft)
{
   // Si le handle de traitement n'est pas initialisé à 0, on ne fait aucun traitement.
   if (isNullHandle(*_hd) || (_hd->data_type != DTREATMENT)) return;
   DataTreatment * const __dt = (DataTreatment*)_hd->data_ptr;
   AllDataTreatment.push_back (__dt);
   switch(*_ft)
   {
      case (NETCDF4):
         __dt->createDataOutput<NetCDF4DataOutput>();
         return;
      // Autres formats de fichiers si disponibles...
      default:
         return;
   }
}

