#ifndef __C_INTERFACE__
#define __C_INTERFACE__

#include "xmlio.hpp"

using namespace XMLIOSERVER;
using namespace XMLIOSERVER::XML;

#ifdef __cplusplus
extern"C"
{
#endif //__cplusplus

   typedef long int XInt; // Integer
   typedef bool XBool;    // Logical
   typedef void *XPtr;    // Pointeur C non typé

   typedef enum _datatype
   {
      NOTYPE = 0,
      DTREATMENT, DATE, CALENDAR, ECONTEXT,
      EAXIS, EDOMAIN, EFIELD, EFILE, EGRID,
      GAXIS, GDOMAIN, GFIELD, GFILE, GGRID
   } XDType;

   typedef enum _filetype
   {
      NETCDF4 = 0
   } XFileType;

   typedef struct _handle // Référence C à une objet de la bibliothèque.
   { XDType data_type; XPtr data_ptr; } XHandle;

   typedef struct _string // Chaîne de caractères.
   { char * str; int len;  } XString;

   typedef struct _ymdhms // Durée et date.
   { int year, month, day, hour, minute, second; } XDate, XDuration;

   const XHandle NULLHANDLE = { NOTYPE, NULL };

#define isNullHandle(hdl) ((hdl).data_type == NOTYPE && (hdl).data_ptr == NULL)

   /* ******************** HANDLE INTERFACE ******************** */
   void xios_handle_create_ (XHandle * _ret, const XDType * const _dtype, XString _id);
   void xios_handle_verify_ (XBool   * _ret, const XHandle * const _hd);
   void xios_handle_getType_(XDType  * _ret, const XHandle * const _hd);

   void xios_handle_isType_   (XBool * _ret, const XHandle * const _hd, const XDType  * const _dtype);
   void xios_handle_isId_     (XBool * _ret, const XHandle * const _hd, const XString * const _id);
   void xios_handle_isGroup_  (XBool * _ret, const XHandle * const _hd);
   void xios_handle_isElement_(XBool * _ret, const XHandle * const _hd);

   /* ******************** XML INTERFACE *********************** */
   void xios_xml_parse_file_  (XString _filename);
   void xios_xml_parse_string_(XString _xmlcontent);

   /* ******************** DATA TREATMENT INTERFACE ************ */
   void xios_dtreatment_new_(XHandle * _dt, const XHandle * const _hd);
   void xios_dtreatment_create_files_and_headers_(const XHandle * const _hd, const XFileType * const _ft);

   /* ******************** CALENDAR INTERFACE ****************** */

   void xios_date_new_(XHandle * _dte, const XDate * const _xdate);
   void xios_calendar_setTimeStep_(XHandle * _cal, const XDuration * const _dur);

   /* ******************** CONTEXT INTERFACE ******************* */
   void xios_context_SetCurrent_(const XHandle * const _ctxt);
   void xios_context_GetCurrent_(XHandle * _ctxt);
   void xios_context_ShowTree_  (void);

   void xios_context_getCalendar_(const XHandle * const _ctxt, XHandle* _cal);

   /* ******************** AXIS INTERFACE ********************** */
   /* ******************** DOMAIN INTERFACE ******************** */
   /* ******************** FIELD INTERFACE ********************* */
   /* ******************** FILE INTERFACE ********************** */
   /* ******************** GRID INTERFACE ********************** */
   /* ******************** GROUP INTERFACE ********************* */

#ifdef __cplusplus
}
#endif //__cplusplus


#endif //__C_INTERFACE__
